from typing import List, Dict, Union

from ldappy.core.lib.ldap import Ldap


class LdapUr(Ldap):
    """ Classe héritée de Ldap avec des méthodes spécifiques pour le Ldap de l'université de la Réunion """

    def get_info_by_matricule(self, matricule: str) -> List[str]:
        """ Récupère toutes les informations ldap par rapport à un matricule (SIHAM) donné.

        Args:
            matricule (str): L'élément qu'on recherche. Dans notre cas, le matricule d'un agent.

        Returns:
            La liste des entrées ldap par rapport à un matricule.
        """
        result = []
        entries = Ldap._generic_search(self, f'runUnivSihamId={matricule}', branch='ou=People,dc=univ-reunion,dc=fr')

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))
        return result

    def get_all_value_specatt(self) -> List[str]:
        result = set()
        entries = Ldap._generic_search(self, f'runUnivAuthorization=*',
                                       branch='ou=People,dc=univ-reunion,dc=fr', attributes=['runUnivAuthorization'])
        for entry in entries:
            authorizations = Ldap.get_entry_json(entry)
            for auhtorization in authorizations['attributes']['runUnivAuthorization']:
                tmp = auhtorization
                result.add(tmp)
        return list(result)

    def get_groups(self) -> List[str]:
        """ Récupère les groupes ldap dans SupannGroupe

        Returns:
            La liste des groupes ldap
        """
        result = []
        entries = Ldap._generic_search(self, 'objectClass=supannGroupe',
                                       branch='ou=Groups,dc=univ-reunion,dc=fr', attributes=None)

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))

        return result

    def get_all_uo(self) -> List[str]:
        """ Récupère les structures (UO)

        Returns:
            Les Unités Organisationnelles sous formes de liste de dictionnaire
        """

        result = []

        entries = Ldap._generic_search(self, 'supannCodeEntite=HU*',
                                       branch='ou=Structures,dc=univ-reunion,dc=fr',
                                       attributes=["supannCodeEntite", "description"])
        for entry in entries:
            result.append(Ldap.get_entry_json(entry))
        return result

    def add_authorization(self, value: List[str], *, dn: str) -> Dict[str, Union[int, str, bool]]:
        """ Ajoute une valeur à l'attribut 'runUnivAuthorization'

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera ajouté à l'attribut 'runUnivAuthorization'
            dn (str): La dn sur laquelle on se place

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        return Ldap._generic_modify_add(self, 'runUnivAuthorization', value, dn=dn)

    def add_authorization_by_matricule(self, value: List[str], *, matricule: str) -> Dict[str, Union[int, str, bool]]:
        """ Ajoute une valeur à l'attribut 'runUnivAuthorization' par rapport à un matricule

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera ajouté à l'attribut 'runUnivAuthorization'
            matricule (str): Le matricule SIHAM de l'agent

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        entries = Ldap._generic_search(self, f'runUnivSihamId={matricule}', branch='ou=People,dc=univ-reunion,dc=fr')
        if len(entries) == 1:
            dn = Ldap.get_entry_json(entries[0])['dn']
            return Ldap._generic_modify_add(self, 'runUnivAuthorization', value, dn=dn)
        elif len(entries) == 0:
            return {'code': 30,
                    'msg': "Matricule n'existe pas",
                    'debug': f"Le matricule {matricule} n'existe pas",
                    'success': False}
        else:
            return {'code': 40,
                    'msg': "Plusieurs entrées existent",
                    'debug': f"Plusieurs entrées existent pour le matricule {matricule}",
                    'success': False}

    def add_authorization_by_login(self, value: List[str], *, login: str) -> Dict[str, Union[int, str, bool]]:
        """ Ajoute une valeur à l'attribut 'runUnivAuthorization' par rapport à un login

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera ajouté à l'attribut 'runUnivAuthorization'
            login (str): Le matricule SIHAM de l'agent

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        entries = Ldap._generic_search(self, f'supannAliasLogin={login}', branch='ou=People,dc=univ-reunion,dc=fr')
        if len(entries) == 1:
            dn = Ldap.get_entry_json(entries[0])['dn']
            return Ldap._generic_modify_add(self, 'runUnivAuthorization', value, dn=dn)
        elif len(entries) == 0:
            return {'code': 30,
                    'msg': "Matricule n'existe pas",
                    'debug': f"Le login {login} n'existe pas",
                    'success': False}
        else:
            return {'code': 40,
                    'msg': "Plusieurs entrées existent",
                    'debug': f"Plusieurs entrées existent pour le login {login}",
                    'success': False}

    def delete_authorization(self, value: List[str], *, dn: str) -> Dict[str, Union[int, str, bool]]:
        """ Supprime une valeur à l'attribut 'runUnivAuthorization'

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera supprimé à l'attribut 'runUnivAuthorization'
            dn (str): La dn sur laquelle on se place

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        return Ldap._generic_modify_delete(self, 'runUnivAuthorization', value, dn=dn)

    def delete_authorization_by_matricule(self, value: List[str], *, matricule: str) -> Dict[str, Union[int, str, bool]]:
        """ Supprime une valeur à l'attribut 'runUnivAuthorization' par rappport à un matricule recherché

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera supprimé à l'attribut 'runUnivAuthorization'
            matricule (str): Le matricule SIHAM de l'agent

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        entries = Ldap._generic_search(self, f'runUnivSihamId={matricule}', branch='ou=People,dc=univ-reunion,dc=fr')
        if len(entries) == 1:
            dn = Ldap.get_entry_json(entries[0])["dn"]
            return Ldap._generic_modify_delete(self, 'runUnivAuthorization', value, dn=dn)
        elif len(entries) == 0:
            return {'code': 30,
                    'msg': "Matricule n'existe pas",
                    'debug': f"Le matricule {matricule} n'existe pas",
                    'success': False}
        else:
            return {'code': 40,
                    'msg': "Plusieurs entrées existent",
                    'debug': f"Plusieurs entrées existent pour le matricule {matricule}",
                    'success': False}

    def delete_authorization_by_login(self, value: List[str], *, login: str) -> Dict[str, Union[int, str, bool]]:
        """ Supprime une valeur à l'attribut 'runUnivAuthorization' par rappport à un login recherché

        Args:
            value (List[str]):  Une liste de chaine de caractère qui sera supprimé à l'attribut 'runUnivAuthorization'
            login (str):     Le login SIHAM de l'agent

        Returns:
            Dictionnaire décrivant le succès ou non de l'opération.
        """
        entries = Ldap._generic_search(self, f'supannAliasLogin={login}', branch='ou=People,dc=univ-reunion,dc=fr')
        if len(entries) == 1:
            dn = Ldap.get_entry_json(entries[0])["dn"]
            return Ldap._generic_modify_delete(self, 'runUnivAuthorization', value, dn=dn)
        elif len(entries) == 0:
            return {'code': 30,
                    'msg': "Matricule n'existe pas",
                    'debug': f"Le login {login} n'existe pas",
                    'success': False}
        else:
            return {'code': 40,
                    'msg': "Plusieurs entrées existent",
                    'debug': f"Plusieurs entrées existent pour le login {login}",
                    'success': False}

    def recup_agent_habsiham(self) -> List:
        """ Récupère le dn des agents qui possède une habilitation dans siham (runUnivAuthorization = siham-hra)


        Returns:
            Liste de dn (agent) qui possède dans son attribut runUnivAuthorization une valeur équivalente à siham-hra:*
        """

        result = []
        entries = Ldap._generic_search(self, 'runUnivAuthorization=siham-hra:*',
                                       branch='ou=People,dc=univ-reunion,dc=fr',
                                       attributes=['runUnivAuthorization', 'runUnivSihamId'])

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))

        return result

    def get_agents(self) -> List:
        """ Récupère le dn des agents qui possède un runUnivSihamId et un mail en '@univ-reunion.fr'
        C'est à dire tous les agents.


        Returns:
            Liste de dn (agent) qui possède dans un attribut runUnivSihamId et un mail en "@univ-reunion.fr"
        """

        result = []
        entries = Ldap._generic_search(self, '&(runUnivSihamId=*)(mail=*@univ-reunion.fr)(eduPersonAffiliation=member)\
                                                (supannAliasLogin=*)',
                                       branch='ou=People,dc=univ-reunion,dc=fr',
                                       attributes=['runUnivNom', 'runUnivPrenom', 'supannAliaslogin',
                                                   'mail', 'runUnivSihamId', 'runUnivAuthorization',
                                                   'supannEntiteAffectationPrincipale', 'runUnivDateFin',
                                                   'runUnivCategorie'])

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))

        return result

    def get_agents_gpfy(self) -> List:
        """ Récupère le dn des agents qui possèdent un runUnivSihamId et un mail en '@univ-reunion.fr'
        C'est-à-dire tous les agents.


        Returns:
            Liste de dn (agent) qui possède dans un attribut runUnivSihamId et un mail en "@univ-reunion.fr"
        """

        result = []
        entries = Ldap._generic_search(self, '&(runUnivSihamId=*)(mail=*@univ-reunion.fr)(eduPersonAffiliation=member)\
                                                (supannAliasLogin=*)(!(runUnivAuthorization=siham-hra:*))',
                                       branch='ou=People,dc=univ-reunion,dc=fr',
                                       attributes=['runUnivNom', 'runUnivPrenom', 'supannAliaslogin',
                                                   'mail', 'runUnivSihamId', 'runUnivAuthorization',
                                                   'supannEntiteAffectationPrincipale'])

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))

        return result

    def get_agents_mail(self) -> List:
        """ Récupère le dn des agents qui possèdent un runUnivSihamId et un mail en '@univ-reunion.fr'
        C'est-à-dire tous les agents.


        Returns:
            Liste de dn (agent) qui possède dans un attribut runUnivSihamId et un mail en "@univ-reunion.fr"
        """

        result = []
        entries = Ldap._generic_search(self, '&(runUnivSihamId=*)(mail=*@univ-reunion.fr)\
                                                (supannAliasLogin=*)',
                                       branch='ou=People,dc=univ-reunion,dc=fr',
                                       attributes=['runUnivNom', 'runUnivPrenom', 'supannAliaslogin',
                                                   'mail', 'runUnivSihamId', 'runUnivAuthorization',
                                                   'supannEntiteAffectationPrincipale'])

        for entry in entries:
            result.append(Ldap.get_entry_json(entry))

        return result

