from dataclasses import dataclass
from configparser import ConfigParser
import pathlib
from ldappy.tools.toolbox import Toolbox
from typing import Dict


@dataclass
class Configuration:

    @classmethod
    def createldp(cls, config_file_path: str, environment_name: str, host: str, port: str, ldap_user: str,
                  ldap_password: str, base: str) -> str:
        """ Création fichier configuration de la connexion au ldap

        Args:
            config_file_path (str): Chemin du fichier de configuration du ldap
            environment_name (str): Nom d'environnement (label) du ldap (exemple: prod, preprod, test, ...)
            host (str): Nom du serveur ldap
            port (str): Port de connexion au ldap
            ldap_user (str): Nom utilisateur ldap
            ldap_password (str): Mot de passe ldap
            base (str): Base cn ldap

        Returns:
            Chemin absolu du fichier de configuration créé
        """
        file = f'{pathlib.Path(__file__).parents[1]}/{config_file_path}.ini'
        usr_bytes = bytes(ldap_user, 'utf-8')
        pwd_bytes = bytes(ldap_password, 'utf-8')
        config = ConfigParser()
        config.optionxform = str

        config.add_section('LDAP')
        config['LDAP']['ENVIRONMENT_NAME'] = environment_name
        config['LDAP']['HOST'] = host
        config['LDAP']['PORT'] = port
        config['LDAP']['LDAP_USER'] = Toolbox.obscure(usr_bytes).decode('utf-8')
        config['LDAP']['LDAP_PASSWORD'] = Toolbox.obscure(pwd_bytes).decode('utf-8')
        config['LDAP']['BASE'] = base

        with open(file, 'w') as configfile:
            config.write(configfile)

        return file

    @staticmethod
    def loadldap(file: str = Toolbox.abs_pathconf_ws()) -> Dict:
        """ Récupère les informations disponibles dans le ficiher de configuration du ldap

        Returns:
            Retourne, sous forme de dictionnaire, les informations contenues dans le fichier de configuration
            du ldap
        """

        configfile = file
        config = ConfigParser()
        config.read(configfile, encoding='utf-8')

        data = {'ENVIRONMENT_NAME': config['LDAP']['ENVIRONMENT_NAME'],
                'HOST': config['LDAP']['HOST'],
                'PORT': config['LDAP']['PORT'],
                'LDAP_USER': config["LDAP"]["LDAP_USER"],
                'LDAP_PASSWORD': config["LDAP"]["LDAP_PASSWORD"],
                'BASE': config["LDAP"]["BASE"]}

        return data

    @staticmethod
    def create_confmod(environment: str, ldap_config_file: str, dist_path: str = 'dist', docs_path: str = 'docs',
                       debug: bool = False, file: str = f"{pathlib.Path(__file__).parents[1]}/config.ini"):
        """ Crée le fichier de base de la configuration du module ldap

        Args:
            environment (str): nom de l'environnement
            ldap_config_file (str): chemin du fichier de configuration WebService ldap associé
            dist_path (str): chemin du dossier DIST (distribution). Par défaut : dist
            docs_path (str): chemin du dossier DOC (documentation). Par défaut : docs
            debug (bool): flag debuggage. Par défaut : False
            file (str): chemin du fichier de configration du module ldap. Par défaut : config.ini

        """

        config = ConfigParser()
        config.optionxform = str

        config.add_section('APP')
        config['APP']['ENVIRONMENT'] = environment
        config['APP']['DEBUG'] = str(debug)
        config['APP']['LDAP_CONFIG_FILE'] = ldap_config_file
        config['APP']['DIST_PATH'] = dist_path
        config['APP']['DOCS_PATH'] = docs_path

        with open(file, 'w+') as configfile:
            config.write(configfile)
